program Fading;
{****************************************************************************
** Demonstration of fading a palette in from black or fading out to black  **
**  by Steven H Don                                                        **
**                                                                         **
** For questions, feel free to e-mail me.                                  **
**                                                                         **
**    shd@earthling.net                                                    **
**    http://shd.cjb.net                                                   **
**                                                                         **
****************************************************************************}

uses Crt;

var
  Palette : Array [0..255, 0..2] of Byte;

{This sets the display to VGA 320x200 in 256 colours}
procedure VgaScreen; assembler;
asm
  mov ax, 13h
  int 10h
end;

{This resets the display to text mode}
procedure TextScreen; assembler;
asm
  mov ax, 3h
  int 10h
end;

{This reads the Red Green Blue-values of a DAC register}
procedure ReadDAC (DAC : Byte; var R, G, B : Byte);
begin
  Port [$3C7] := DAC;
  R := Port [$3C9];
  G := Port [$3C9];
  B := Port [$3C9];
end;

{This sets a DAC register to a specific Red Green Blue-value}
procedure SetDAC (DAC, R, G, B : Byte);
begin
  Port [$3C8] := DAC;
  Port [$3C9] := R;
  Port [$3C9] := G;
  Port [$3C9] := B;
end;

{This sets all the DAC registers to black}
procedure BlackDACs;
var
  Counter : Byte;

begin
  for Counter := 0 to 255 do
    SetDAC (Counter, 0, 0, 0);
end;

{This fades in the palette as stored in the "Palette" array.}
procedure FadeIn;
var
  Counter, R, G, B : Byte;
  Done : Boolean;

begin
  repeat
    {Assume fading is completed}
    Done := true;
    {Adjust all 256 DAC registers}
    for Counter := 0 to 255 do begin
      {Read the DAC register}
      ReadDAC (Counter, R, G, B);
      {Check to see whether Red component has reached its final level}
      if R < Palette [Counter, 0] then begin
        {if not, increase it by one...}
        inc (R);
        {... and note that the fade is not complete yet}
        Done := false;
      end;
      {Do the same for the Green component}
      if G < Palette [Counter, 1] then begin
        inc (G);
        Done := false;
      end;
      {Do the same for the Blue component}
      if B < Palette [Counter, 2] then begin
        inc (B);
        Done := false;
      end;
      {Set the DAC register to the new values}
      SetDAC (Counter, R, G, B);
    end;
    {Slow down the fade}
    Delay (10);
  until Done;
end;

{This fades out the on-screen palette.}
procedure FadeOut;
var
  Counter, R, G, B : Byte;
  Done : Boolean;

begin
  repeat
    {Assume fading is completed}
    Done := true;
    {Adjust all 256 DAC registers}
    for Counter := 0 to 255 do begin
      {Read the DAC register}
      ReadDAC (Counter, R, G, B);
      {Check to see whether Red component has reached 0}
      if R > 0 then begin
        {if not, decrease it by one...}
        dec (R);
        {... and note that the fade is not complete yet}
        Done := false;
      end;
      {Do the same for the Green component}
      if G > 0 then begin
        dec (G);
        Done := false;
      end;
      {Do the same for the Blue component}
      if B > 0 then begin
        dec (B);
        Done := false;
      end;
      {Set the DAC register to the new values}
      SetDAC (Counter, R, G, B);
    end;
    {Slow down the fade}
    Delay (10);
  until Done;
end;

{This sets up a smooth blue palette}
procedure BluePalette;
var
  Counter : Byte;

begin
  for Counter := 0 to 255 do begin
    {Set up a palette...}
    {No red}
    Palette [Counter, 0] := 0;
    {No green either}
    Palette [Counter, 1] := 0;
    {But increasingly bright blue}
    Palette [Counter, 2] := Counter shr 2;
  end;
end;

{This displays a smooth pattern on screen, just for demonstration purposes}
procedure Gradient;
var
  Offset : Word;

begin
  for OffSet := 0 to 63999 do
    Mem [$A000:OffSet] := OffSet mod 315;
end;

begin
  {Display message}
  Writeln ('Fading demonstration');
  Writeln ('Press any key to fade out and switch to VGA graphics mode...');
  {Wait for keypress}
  ReadKey;
  {Fade out - note: it also works in textmode!}
  FadeOut;
  {Switch to VGA}
  VgaScreen;
  {Clear the on-screen palette}
  BlackDACs;
  {Set the "Palette" array to a smooth blue}
  BluePalette;
  {Display a pattern on-screen}
  Gradient;
  {Fade in the blue}
  FadeIn;
  {Wait for a keypress}
  ReadKey;
  {Fade out again}
  FadeOut;
  {And finally, return to text}
  TextScreen;
end.